<?php 

/**
 * A PHP version of TinyMCE's configuration, to allow various parameters to be configured on a site or section basis
 * 
 * There can be multiple HtmlEditorConfig's, which should always be created / accessed using HtmlEditorConfig::get.
 * You can then set the currently active config using set_active. Whichever config is active when
 * HtmlEditorField#Field is called wins.
 *  
 * @author "Tony Air" <tony@thewebdevelopmentagency.com>
 * @package forms
 * @subpackage fields-formattedinput
 */
class ElEditorConfig {

	static $configs = array();
	static $current = null;
	public static $ver = '';
	
	/**
	 * Get the HtmlEditorConfig object for the given identifier. This is a correct way to get an HtmlEditorConfig
	 * instance - do not call 'new'
	 * 
	 * @param $identifier string - the identifier for the config set
	 * @return HtmlEditorConfig - the configuration object. This will be created if it does not yet exist for that
	 *                            identifier
	 */
	public static function get($identifier = 'default') {
		if (!array_key_exists($identifier, self::$configs)) self::$configs[$identifier] = new ElEditorConfig();
		return self::$configs[$identifier];
	}
	
	/**
	 * Set the currently active configuration object
	 * @param $identifier string - the identifier for the config set
	 * @return null
	 */
	public static function set_active($identifier = null) {
		self::$current = $identifier;
	}
	
	/**
	 * Get the currently active configuration object
	 * @return HtmlEditorConfig - the active configuration object
	 */
	public static function get_active() {
		$identifier = self::$current ? self::$current : 'default';
		return self::get($identifier);
	}
	
	/**
	 * Get the available configurations as a map of friendly_name to
	 * configuration name.
	 * @return array
	 */
	public static function get_available_configs_map() {
		$configs = array();
		
		foreach(self::$configs as $identifier => $config) {
			$configs[$identifier] = $config->getOption('friendly_name');
		}
		
		return $configs;
	}
	
	/**
	 * Holder for all TinyMCE settings _except_ plugins and buttons
	 */
	protected $settings = array(
		'absoluteURLs'	=>	true,
		'toolbar'		=>	'SSEditorToolbar', // $panelName.'Toolbar'
		'lang'			=>	'en', // set your language
		'cssClass'		=>	'el-rte',
		'styleWithCSS'	=>	false,
		'cssfiles'		=>	array(),
		'height'		=>	100,
	);

	/**
	 * Holder list of buttons, organised by line
	 */
	protected $panels = array(
		'basic' => array(
			'pastetext','pasteformattext','removeformat','undo','redo',
			'bold','italic','underline','strikethrough','subscript','superscript','forecolor',
			'justifyleft','justifycenter','justifyright','justifyfull',
			'outdent','indent','css','nbsp','link','unlink','image','fullscreen'
			//'ltr', 'rtl'
		),
		'format' => array(
			'formatblock','fontsize','fontname',
			'insertorderedlist','insertunorderedlist','blockquote',
			'div','stopfloat',//'pagebreak',
		),
		'tables' => array(
			'table','tableprops','tablerm','tbrowbefore','tbrowafter',
			'tbrowrm','tbcolbefore','tbcolafter','tbcolrm','tbcellprops',
			'tbcellsmerge', 'tbcellsplit',
		)
	);

	/**
	 * Get the current value of an option
	 * @param $k string - The key of the option to get
	 * @return mixed - The value of the specified option 
	 */
	public function getOption($k) {
		if(isset($this->settings[$k])) return $this->settings[$k];
	}
	
	/**
	 * Set the value of one option
	 * @param $k string - The key of the option to set
	 * @param $v mixed - The value of the option to set
	 * @return mixed - $v returned for chaining
	 */
	public function setOption($k,$v) {
		$this->settings[$k] = $v;
		return $this;
	}
	
	/**
	 * Set multiple options
	 * @param $a array - The options to set, as keys and values of the array
	 * @return null
	 */
	public function setOptions($a) {
		foreach ($a as $k=>$v) {
			$this->settings[$k] = $v;
		}
		return $this;
	}

	/**
	 * Generate the javascript that will set tinyMCE's configuration to that of the current settings of this object
	 * @return string - the javascript
	 */
	public function generateJS() {
		$config = $this->settings;
		$config['baseURL'] = Director::absoluteBaseURL();
		$config['lang'] = i18n::get_tinymce_lang();
		if(!$config['cssfiles']){
			$config['cssfiles'] = array(
				Director::absoluteBaseURL().ZZ_BASIC_PATH.'/backend/css/editor.css',
				Director::absoluteBaseURL().ZZ_BASIC_PATH.'/javascript/bootstrap/css/bootstrap.css',
				Director::absoluteBaseURL().ZZ_BASIC_PATH.'/css/layout.css',
				Director::absoluteBaseURL().ZZ_BASIC_PATH.'/css/typography.css',
				Director::absoluteBaseURL().project().'/css/layout.css',
				Director::absoluteBaseURL().project().'/css/typography.css'
			);
		}
		// panels
		$panels = array();
		$panelOutput = '';
		foreach($this->panels as $name => $buttons) {
			$panelOutput .= 'elRTE.prototype.options.panels.'.$name.' = '.Convert::raw2json($buttons).';';
			$panels[] = $name;
		}
		$panelOutput .= 'elRTE.prototype.options.toolbars.SSEditorToolbar = '.Convert::raw2json($panels).';';
		//Convert::raw2json($config)$externalPluginsJS
		$initScript = 'if((typeof elRTE != "undefined")){'.$panelOutput.' var ssElRTEAreaConfig = '.Convert::raw2json($config).';ss.editorWrappers[\'default\'] = ss.editorWrappers.ElEditor;}';
		// create editor
		// $("textarea.elRTEArea").elrte(elRTEAreaConfig);
		// var editor = new elRTE(document.getElementById("our-element"), opts);
		return $initScript;
	}
}
